@@ -1,18 +1,19 @@
 """Miscellaneous utility functions"""
 import copy
 import os
-import random
 import re
 import shlex
 import sys
 import tempfile
 import time
 import traceback
+import urllib.parse
 
 from . import core
 from . import compat
 
-random.seed(hash(time.time()))
+
+_SSH_REGEX = re.compile(r'^(?P<user>[^@]+)@(?P<hostname>[^:]+):(?P<path>.+)')
 
 
 def asint(obj, default=0):
@@ -29,6 +30,11 @@ def clamp(value, low, high):
     return min(high, max(low, value))
 
 
+def clamp_zero(value, maximum):
+    """Clamp a value between 0 and the maximum value"""
+    return clamp(value, 0, maximum)
+
+
 def epoch_millis():
     return int(time.time() * 1000)
 
@@ -311,7 +317,7 @@ def launch_default_app(paths):
     if is_win32():
         for path in paths:
             if hasattr(os, 'startfile'):
-                os.startfile(path)
+                os.startfile(os.path.abspath(path))
         return
 
     if is_darwin():
@@ -330,6 +336,79 @@ def expandpath(path):
     return path
 
 
+def get_hostname_from_url(url_str):
+    """Extract the hostname component from a URL
+
+    >>> get_hostname_from_url('user@example.org:namespace/project')
+    'example.org'
+
+    >>> get_hostname_from_url('user@example.org:namespace/project.git/')
+    'example.org'
+
+    >>> get_hostname_from_url('https://example.org/namespace/project.git')
+    'example.org'
+
+    >>> get_hostname_from_url('ssh://user@example.org/namespace/project.git')
+    'example.org'
+
+    """
+    if not url_str:
+        return None
+    url = urllib.parse.urlparse(url_str)
+    if not url or url.scheme == '' or url.netloc == '':
+        # Handle <user>@<host>:path.git URLs
+        match = _SSH_REGEX.match(url_str)
+        if match is None:
+            return None
+        hostname = match.group('hostname')
+        if not hostname:
+            return None
+    else:
+        hostname = url.netloc
+
+    if '@' in hostname:
+        _, hostname = hostname.split('@', 1)
+
+    return hostname
+
+
+def get_path_from_url(url_str):
+    """Extract the path component from a URL
+
+    >>> get_path_from_url('user@example.org:namespace/project')
+    'namespace/project'
+
+    >>> get_path_from_url('user@example.org:namespace/project.git/')
+    'namespace/project'
+
+    >>> get_path_from_url('https://example.org/namespace/project.git')
+    'namespace/project'
+
+    >>> get_path_from_url('ssh://user@example.org/namespace/project.git')
+    'namespace/project'
+
+    """
+    if not url_str:
+        return None
+    url = urllib.parse.urlparse(url_str)
+    if not url or url.scheme == '' or url.path == '':
+        # Handle <user>@<host>:path.git URLs
+        match = _SSH_REGEX.match(url_str)
+        if match is None:
+            return None
+        path = match.group('path')
+        if not path:
+            return None
+    else:
+        path = url.path
+
+    path = path.strip('/')  # Strip leading and trailing slashes.
+    if path.endswith('.git'):
+        path = path[: -len('.git')]
+
+    return path
+
+
 class Group:
     """Operate on a collection of objects as a single unit"""
 
@@ -348,6 +427,38 @@ class Group:
         return relay
 
 
+def strip_prefixes_and_suffixes(values, prefix, suffix):
+    """Strip prefixes and suffixes from a sequence of values
+
+    Values are assumed to begin and end with the specified prefix and suffix.
+    """
+    prefix_len = len(prefix)
+    suffix_len = len(suffix)
+    if suffix_len == 0:
+        return sorted(name[prefix_len:] for name in values)
+    return sorted(name[prefix_len:-suffix_len] for name in values)
+
+
+def strip_prefixes_and_suffixes_from_keys(values, prefix, suffix):
+    """Transform dictionary keys to remove prefixes and suffixes
+
+    Values are assumed to begin and end with the specified prefix and suffix.
+    """
+    prefix_len = len(prefix)
+    suffix_len = len(suffix)
+    if suffix_len == 0:
+        return {key[prefix_len:]: value for (key, value) in values.items()}
+    return {key[prefix_len:-suffix_len]: value for (key, value) in values.items()}
+
+
+def strip_prefixes_from_keys(values, prefix):
+    """Transform dictionary keys to remove prefixes
+
+    Values are assumed to begin with the specified prefix.
+    """
+    return strip_prefixes_and_suffixes_from_keys(values, prefix, '')
+
+
 class Proxy:
     """Wrap an object and override attributes"""
 
